//
//  ViewControllerEstris.m
//  ataula
//
//  Created by Andreu Biosca on 02/04/2020.
//  Copyright © 2020 EGDBE. All rights reserved.
//

#import "ViewControllerEstris.h"
#import "IncludesCompartits.h"
#import "ViewControllerEstri.h"
#import "TabBarControllerPrincipal.h"

@interface ViewControllerEstris ()

@end

@implementation ViewControllerEstris

-(void)viewDidAppear:(BOOL)animated
{
    
    [super viewDidAppear:animated];
    
    TabBarControllerPrincipal *oTabBarControllerPrincipal = (TabBarControllerPrincipal*)self.tabBarController;
    
    [oTabBarControllerPrincipal pintaTextosIIcones];
    
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.title = @"Estris";

    // Creem el botó d'afegir una nou estri
    
    UIBarButtonItem *botoAfegir = [[UIBarButtonItem alloc] initWithTitle:@"Afegir" style:UIBarButtonItemStylePlain target:self action:@selector(clicAfegir:)];
    botoAfegir.tintColor = UIColorFromRGB(PALETA_COLOR_VERMELL);
    self.navigationItem.rightBarButtonItem = botoAfegir;
    
    // Cerquem estris (en aquest cas sense filtres, es a dir, tots)
    
    [self cercaEstris:@""];
    
}

-(void)cercaEstris:(NSString*)sCerca
{
    
    self.iIdFilCercaValid++;
    
    __block int iIdFilCercaValid_valid = self.iIdFilCercaValid;
    
    AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
    
    NSString *URL_WS = [NSString stringWithFormat:@"%@/API/cercaEstris",WS_HOST];
    
    NSDictionary *parameters = @{
        @"cerca": sCerca
    };
    
    [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        
        // OK
        
        if(iIdFilCercaValid_valid == self.iIdFilCercaValid){
            
            NSDictionary *json = responseObject;
            
            self.aInstancies = [[NSMutableArray alloc] init];
            
            for (NSDictionary *dict in json[@"data"]) {
                
                [self.aInstancies addObject:dict];
                
            }
            
            if(iIdFilCercaValid_valid == self.iIdFilCercaValid)[self.tableview reloadData];

        }
        
    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        
        // KO
        
        [self presentViewController:[IncludesCompartits creaAlerta:@"Error"
                                                        descripcio:MISSATGE_ERROR_DEFECTE_WS] animated:YES completion:nil];
        
    }];
    
}


-(void)recercaEstris
{
    
    [self cercaEstris:self.searchbar.text];
    
}

- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText
{

    [self cercaEstris:searchText];
    
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar
{
    
    [searchBar resignFirstResponder];
    
}

-(void)clicAfegir:(id)sender
{

    if([self.searchbar isFirstResponder])[self.searchbar resignFirstResponder];
    
    ViewControllerEstri *oViewControllerEstri = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"viewControllerEstri"];
    oViewControllerEstri.oViewControllerEstris = self;
    [self.navigationController pushViewController:oViewControllerEstri animated:YES];
  
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    
    if([self.searchbar isFirstResponder])[self.searchbar resignFirstResponder];
    
}

#pragma mark Tableview delegate

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    return 49;
    
}

- (UITableViewCell *)tableView:(UITableView *)aTableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    float amplada_pantalla = [[UIScreen mainScreen] bounds].size.width;
    
    NSString *sReuseIdentifier = @"Estri_cell_reuse_identifier";
    
    UITableViewCell *cell = [aTableView dequeueReusableCellWithIdentifier:sReuseIdentifier];
    
    // Càlculs comuns
    
    NSDictionary *dictEntitat = self.aInstancies[indexPath.row];
    
    NSString *titol = dictEntitat[@"nom"];
    
    int iWidth_labels = amplada_pantalla-30;
    
    NSMutableAttributedString * atributtedString_TITOL = [[NSMutableAttributedString alloc] initWithString:titol];
    [atributtedString_TITOL addAttribute:NSFontAttributeName value:[UIFont systemFontOfSize:16] range: NSMakeRange(0,[titol length])];
    [atributtedString_TITOL addAttribute:NSForegroundColorAttributeName value:[UIColor blackColor] range: NSMakeRange(0,[titol length])];
    
    if(cell == nil)
    {
        
        //NSLog(@"CELL = NIL");
        
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:sReuseIdentifier];
        
        UILabel * labelcell_TITOL = [[UILabel alloc]initWithFrame:CGRectMake(14, 4, iWidth_labels, 40)];
        labelcell_TITOL.attributedText = atributtedString_TITOL; // ?¿
        labelcell_TITOL.numberOfLines = 2;
        labelcell_TITOL.backgroundColor = UIColor.clearColor;
        labelcell_TITOL.tag = 1;
        [cell.contentView addSubview:labelcell_TITOL];
        
    }else{
        
        //NSLog(@"CELL <> NIL");
        
        for (UIView * subview in cell.contentView.subviews) {
            
            if([subview class] == [UILabel  class])
            {
                
                if(subview.tag == 1)((UILabel*)subview).attributedText = atributtedString_TITOL;
                
            }

        }
        
    }
    
    return cell;
    
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    
    return 1;
    
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    if(self.aInstancies == nil)return 0;
    
    return self.aInstancies.count;
    
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    [cell setBackgroundColor:[UIColor clearColor]];
    
}

- (void)tableView: (UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    if([self.searchbar isFirstResponder])[self.searchbar resignFirstResponder];
    
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
    
    NSDictionary *dictEntitat = self.aInstancies[indexPath.row];
    ViewControllerEstri *oViewControllerEstri = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"viewControllerEstri"];
    oViewControllerEstri.idEstri = dictEntitat[@"id"];
    oViewControllerEstri.oViewControllerEstris = self;
    [self.navigationController pushViewController:oViewControllerEstri animated:YES];
    
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {

    return YES;

}

- (NSString *)tableView:(UITableView *)tableView titleForDeleteConfirmationButtonForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    return @"Eliminar";
    
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle
forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        
        NSDictionary *dictEntitat = self.aInstancies[indexPath.row];
        
        if([self.searchbar isFirstResponder])[self.searchbar resignFirstResponder];

        bool bEsiPad = [[UIDevice currentDevice].model rangeOfString:@"iPad"].location != NSNotFound;
        
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Confirmar"
                                                                       message:[NSString stringWithFormat:@"Estàs segur que vols eliminar %@?", [IncludesCompartits retornaParaulaAmbArticle:dictEntitat[@"genere"] numero:@"S" paraula:dictEntitat[@"nom"]]]
                                                                preferredStyle:bEsiPad ? UIAlertControllerStyleAlert : UIAlertControllerStyleActionSheet];
        
        UIAlertAction *eliminarAccio = [UIAlertAction actionWithTitle:@"Elimina estri"
                                                               style:UIAlertActionStyleDestructive handler:^(UIAlertAction * action) {
            
            AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
            [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
            
            NSString *URL_WS = [NSString stringWithFormat:@"%@/API/eliminaEstri",WS_HOST];
            
            NSDictionary *parameters = @{
                @"id_estri": dictEntitat[@"id"]
            };
            
            [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
                
                // OK
                
                [self recercaEstris];
                
            } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
                
                // KO
                [self presentViewController:[IncludesCompartits creaAlerta:@"Error"
                                                                descripcio:MISSATGE_ERROR_DEFECTE_WS] animated:YES completion:nil];
                
            }];
            
        }];
        
        [alert addAction:eliminarAccio];
        
        [alert addAction:[UIAlertAction actionWithTitle:@"Cancel·la" style:UIAlertActionStyleCancel handler:^(UIAlertAction * action) {}]];
        
        [self presentViewController:alert animated:YES completion:nil];
        
    }
    
}

@end
