//
//  ViewControllerRecepta.m
//  ataula
//
//  Created by Andreu Biosca on 07/04/2020.
//  Copyright © 2020 EGDBE. All rights reserved.
//

#import "ViewControllerRecepta.h"
#import "IncludesCompartits.h"
#import "TabBarControllerPrincipal.h"
#import "ViewControllerReceptes.h"
#import "ViewControllerAfegirIngredientRecepta.h"
#import "VistaMedallaPosicio.h"
#import "ViewControllerPas.h"
#import "JLSlug.h"

@interface ViewControllerRecepta ()

@end

@implementation ViewControllerRecepta

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    // Creació precàrrrega
    
    // Vista preloader:
    self.vista_preloader = [IncludesCompartits creaVistaPreloader:self];
    self.vista_preloader.hidden = YES;
    
    [IncludesCompartits navigationBarAfegirBotoDreta:self imatge:[UIImage imageNamed:@"share.png"] size:CGSizeMake(20, 20) action:@selector(clickCompartirRecepta:) target:self];
    
    self.title = @"Recepta";
    self.navigationController.navigationBar.tintColor = UIColorFromRGB(PALETA_COLOR_VERMELL);
    
    AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
    
    NSString *URL_WS = [NSString stringWithFormat:@"%@/API/getRecepta",WS_HOST];
    
    NSDictionary *parameters = @{
        @"id": self.idRecepta
    };
    
    [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        
        // OK
        
        NSDictionary *json = responseObject;
        
        if([json[@"result"] intValue] == 1)
        {
            
            // KO
            [self.navigationController popViewControllerAnimated:YES];
            
        }else{
            
            NSDictionary *dictEntitat = json[@"data"];
            
            // Bloc capçalera
            
            self.textfield_titol.text = dictEntitat[@"titol"];
            self.textfield_preu.text =  [dictEntitat[@"preu"] isEqual:[NSNull null]] ? @"" : [NSString stringWithFormat:@"%@ €", dictEntitat[@"preu"]];
            self.textfield_temps.text =  [dictEntitat[@"temps"] isEqual:[NSNull null]] ? @"" : [NSString stringWithFormat:@"%@ minuts", dictEntitat[@"temps"]];
            
            NSString *url_imatge = [dictEntitat[@"foto"] isEqualToString:@""] ? @"" : [NSString stringWithFormat:@"%@%@",WS_HOST,dictEntitat[@"foto"]];
            
            if([url_imatge isEqualToString:@""])
            {
                
                self.camera.hidden = NO;
                
            }else{
                
                dispatch_queue_t thread_CARREGAFOTO=dispatch_queue_create("thread_CARREGAFOTO", NULL);
                dispatch_async(thread_CARREGAFOTO, ^{
                    
                    // WS, TASCA LLARGA,...
                    NSData * imgdata = [NSData dataWithContentsOfURL:[NSURL URLWithString:url_imatge]];
                    
                    dispatch_async(dispatch_get_main_queue(), ^{
                        
                        // UI CHANGES
                        if (imgdata == nil)
                        {
                            self.camera.hidden = NO;
                        }else{
                            [self refrescaFoto:[UIImage imageWithData:imgdata]];
                        }
                        
                    });
                    
                });
                
            }
            
            // Ingredients
            
            [self refrescaIngredientsRecepta];
            
            // Passos
            
            [self refrescaPassosRecepta];
            
            // Activem botons
            
            self.botoAfegirPas.enabled = YES;
            self.botoAfegirIngredient.enabled = YES;
            
        }
        
    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        
        // KO
        [self.navigationController popViewControllerAnimated:YES];
        
    }];
    
}

-(void)refrescaPassosRecepta
{
    
    AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
    
    NSString *URL_WS = [NSString stringWithFormat:@"%@/API/getPassosRecepta",WS_HOST];
    
    NSDictionary *parameters = @{
        @"id_recepta": self.idRecepta
    };
    
    [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        
        // OK
        
        NSDictionary *json = responseObject;
        
        self.aInstancies_passos = [[NSMutableArray alloc] init];
        
        for (NSDictionary *dict in json[@"data"]) {
            
            [self.aInstancies_passos addObject:dict];
            
        }
        
        [self.tableview_passos reloadData];
        
        // SIMULAKRE
        // [self clickAfegirPas:nil];

    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        
        // KO
        [self.navigationController popViewControllerAnimated:YES];
        
    }];
    
    
    
}

-(void)refrescaIngredientsRecepta
{
    
    AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
    
    NSString *URL_WS = [NSString stringWithFormat:@"%@/API/getIngredientsRecepta",WS_HOST];
    
    NSDictionary *parameters = @{
        @"id_recepta": self.idRecepta
    };
    
    [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        
        // OK
        
        NSDictionary *json = responseObject;
        
        self.aInstancies_ingredients = [[NSMutableArray alloc] init];
        
        for (NSDictionary *dict in json[@"data"]) {
            
            [self.aInstancies_ingredients addObject:dict];
            
        }
        
        [self.tableview_ingredients reloadData];

    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        
        // KO
        [self.navigationController popViewControllerAnimated:YES];
        
    }];
    
}

- (IBAction)clickCompartirRecepta:(id)sender {
    
    // http://192.168.1.178/recepta/16/arros-amb-pollastre;
    
    NSString *sSlug = [NSString stringWithFormat:@"%@/recepta/%@/%@",WS_HOST,self.idRecepta,[JLSlug str_slug:self.textfield_titol.text]];
    
    UIPasteboard *pasteboard = [UIPasteboard generalPasteboard];
    pasteboard.string = sSlug;
    
    [self presentViewController:[IncludesCompartits creaAlerta:@"Atenció" descripcio:@"S'ha copiat l'enllaç web de la recepta"] animated:YES completion:nil];
    
}

- (IBAction)clickSeleccionarFotografia:(id)sender {
    
    if([self.textfield_preu isFirstResponder]){[self.textfield_preu resignFirstResponder];return;}
    if([self.textfield_titol isFirstResponder]){[self.textfield_titol resignFirstResponder];return;}
    if([self.textfield_temps isFirstResponder]){[self.textfield_temps resignFirstResponder];return;}

    bool bEsiPad = [[UIDevice currentDevice].model rangeOfString:@"iPad"].location != NSNotFound;
    
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil
                                                                   message:nil
                                                            preferredStyle:bEsiPad ? UIAlertControllerStyleAlert : UIAlertControllerStyleActionSheet];
    
    UIAlertAction *ferFotoAccio = [UIAlertAction actionWithTitle:@"Fer foto"
                                                           style:UIAlertActionStyleDefault handler:^(UIAlertAction * action) {
        
        UIImagePickerController * pickerLibrary = [[UIImagePickerController alloc] init];
        pickerLibrary.sourceType = UIImagePickerControllerSourceTypeCamera;
        pickerLibrary.delegate = self;
        [self presentViewController:pickerLibrary animated:YES completion:^{}];
        
    }];
    
    [alert addAction:ferFotoAccio];
    
    UIAlertAction *seleccionarFotoAccio = [UIAlertAction actionWithTitle:@"Seleccionar foto del rodet"
                                                                   style:UIAlertActionStyleDefault handler:^(UIAlertAction * action) {
        
        UIImagePickerController * pickerLibrary = [[UIImagePickerController alloc] init];
        pickerLibrary.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
        pickerLibrary.delegate = self;
        [self presentViewController:pickerLibrary animated:YES completion:^{}];

    }];
    
    [alert addAction:seleccionarFotoAccio];
    
    if((self.imgvFoto != nil) && self.imgvFoto.image != nil)
    {
        
        [alert addAction:[UIAlertAction actionWithTitle:@"Mostrar foto actual" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action) {
            
            self.aTempDocumentURL = [[NSMutableArray alloc]init];
            [self.aTempDocumentURL addObject:[NSURL fileURLWithPath:[NSTemporaryDirectory() stringByAppendingPathComponent:@"Recepta.jpg"]]];
            [UIImageJPEGRepresentation(self.imgvFoto.image, 1) writeToURL:self.aTempDocumentURL[0] atomically:NO];
            
            QLPreviewController *oQLPreviewController = [[QLPreviewController alloc]init];
            oQLPreviewController.dataSource = self;
            oQLPreviewController.delegate = self;
            oQLPreviewController.modalPresentationStyle = UIModalPresentationFullScreen;
            oQLPreviewController.currentPreviewItemIndex = 0;
            
            [self presentViewController:oQLPreviewController animated:YES completion:NULL];
            
        }]];
        
    }
    
    
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel·la" style:UIAlertActionStyleCancel handler:^(UIAlertAction * action) {}]];
    
    [self presentViewController:alert animated:YES completion:nil];
    
}

#pragma mark QuickLook data source

- (id)previewController:(QLPreviewController *)previewController previewItemAtIndex:(NSInteger)index
{
    
    return self.aTempDocumentURL[index];
    
}

- (NSInteger)numberOfPreviewItemsInPreviewController:(QLPreviewController *)controller
{
    
    return self.aTempDocumentURL.count;
    
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<UIImagePickerControllerInfoKey, id> *)info;
{
    
    [self dismissViewControllerAnimated:YES completion:^{
        
        self.vista_preloader.hidden = NO;
        [self.navigationItem setHidesBackButton:YES animated:YES];
        
        UIImage *image = [info objectForKey:UIImagePickerControllerOriginalImage];
        
        [self refrescaFoto:image];
        
        NSData *dataNovaImatge = UIImageJPEGRepresentation([IncludesCompartits getResizedImage:image iMaxWidth:800 iMaxHeight:800], 0.8);
        
        // Actualitzem fotografia
        
        dispatch_queue_t thread_UPLOADFOTO=dispatch_queue_create("thread_UPLOADFOTO", NULL);
        dispatch_async(thread_UPLOADFOTO, ^{

            NSString *boundary = @"---------------------------14737809831466499882746641449";
            
            NSMutableData *body = [NSMutableData data];
            
            // Paràmetres
            
            NSDictionary *params = @{@"id_recepta" : self.idRecepta};
            
            [params enumerateKeysAndObjectsUsingBlock:^(NSString *parameterKey, NSString *parameterValue, BOOL *stop) {
                [body appendData:[[NSString stringWithFormat:@"--%@\r\n", boundary] dataUsingEncoding:NSUTF8StringEncoding]];
                [body appendData:[[NSString stringWithFormat:@"Content-Disposition: form-data; name=\"%@\"\r\n\r\n", parameterKey] dataUsingEncoding:NSUTF8StringEncoding]];
                [body appendData:[[NSString stringWithFormat:@"%@\r\n", parameterValue] dataUsingEncoding:NSUTF8StringEncoding]];
            }];
            
            NSMutableURLRequest *request = [[NSMutableURLRequest alloc] init];
            
            [request setURL:[NSURL URLWithString:[NSString stringWithFormat:@"%@/API/carregarFotoRecepta", WS_HOST]]];
            [request setHTTPMethod:@"POST"];
            
            NSString *contentType = [NSString stringWithFormat:@"multipart/form-data; boundary=%@",boundary];
            [request addValue:contentType forHTTPHeaderField: @"Content-Type"];
            
            [body appendData:[[NSString stringWithFormat:@"\r\n--%@\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
            int num_rnd = arc4random() % 1000;
            [body appendData:[[NSString stringWithFormat:@"Content-Disposition: form-data; name=\"attachments\"; filename=\"%i.jpg\"\r\n",num_rnd] dataUsingEncoding:NSUTF8StringEncoding]];
            [body appendData:[@"Content-Type: application/octet-stream\r\n\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
            [body appendData:[NSData dataWithData: dataNovaImatge]];
            [body appendData:[[NSString stringWithFormat:@"\r\n--%@--\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
            
            [request setHTTPBody:body];
            NSData *dataResponse = [NSURLConnection sendSynchronousRequest:request returningResponse:nil error:nil];
            
            dispatch_async(dispatch_get_main_queue(), ^{
                
                self.vista_preloader.hidden = YES;
                [self.navigationItem setHidesBackButton:NO animated:YES];
                
                if(dataResponse)
                {
                    
                    NSDictionary *json = [NSJSONSerialization JSONObjectWithData:dataResponse options:kNilOptions error:nil];
                    
                    if([json[@"result"] intValue] == 0)
                    {
                        
                        
                        NSString *url_imatge = [NSString stringWithFormat:@"%@/Imatges_elements/Receptes/%@_thumb.jpg",WS_HOST,self.idRecepta];
                        
                        NSLog(@"url_imatge=%@",url_imatge);
                        
                        TabBarControllerPrincipal *oTabBarControllerPrincipal = (TabBarControllerPrincipal*)self.tabBarController;
                        
                        [oTabBarControllerPrincipal.dictImatgesReceptesCollectionCachejades removeObjectForKey:url_imatge];
                        
                        [self.oViewControllerReceptes recercaReceptes];
                        
                    }
                    
                }
                
            });
            
        });
        
        
    }];
    
}

- (IBAction)clickUlleresIngredients:(id)sender {
    
    [self.vista_bloc_ingredients.superview bringSubviewToFront:self.vista_bloc_ingredients];
    
    CGRect rectVB_passos = self.vista_bloc_passos.frame;
    CGRect rectVB_ingredients = self.vista_bloc_ingredients.frame;
    
    bool bBlocMaximitxat = rectVB_ingredients.size.height == self.vista_2_blocs.frame.size.height;
    
    if(!bBlocMaximitxat)
    {
        
        CGRect rectVB_ingredients_final = CGRectMake(0, rectVB_ingredients.origin.y, rectVB_ingredients.size.width, self.vista_2_blocs.frame.size.height);

        self.boto_ulleres_passos.enabled = NO;
        self.boto_ulleres_ingredients.enabled = NO;
        self.liniaUltrafinaSeparadoraBlocs.hidden = YES;
        
        // Animació
        [UIView animateWithDuration:0.5
                              delay:0
                            options:UIViewAnimationOptionCurveEaseInOut animations:^{
            
            self.vista_bloc_ingredients.frame = rectVB_ingredients_final;

        } completion:^(BOOL finished) {
            
            self.boto_ulleres_passos.enabled = YES;
            self.boto_ulleres_ingredients.enabled = YES;
            
        }];

    }else{

        CGRect rectVB_ingredients_final = CGRectMake(0, rectVB_ingredients.origin.y, rectVB_ingredients.size.width, self.vista_2_blocs.frame.size.height - rectVB_passos.size.height);

        self.boto_ulleres_passos.enabled = NO;
        self.boto_ulleres_ingredients.enabled = NO;
        
        // Animació
        [UIView animateWithDuration:0.5
                              delay:0
                            options:UIViewAnimationOptionCurveEaseInOut animations:^{
            
            self.vista_bloc_ingredients.frame = rectVB_ingredients_final;

        } completion:^(BOOL finished) {
            
            self.liniaUltrafinaSeparadoraBlocs.hidden = NO;
            self.boto_ulleres_passos.enabled = YES;
            self.boto_ulleres_ingredients.enabled = YES;
            
        }];
        
    }
    
}

- (IBAction)clickUlleresPassos:(id)sender {
    
    [self.vista_bloc_passos.superview bringSubviewToFront:self.vista_bloc_passos];
    
    CGRect rectVB_passos = self.vista_bloc_passos.frame;
    CGRect rectVB_ingredients = self.vista_bloc_ingredients.frame;
    bool bBlocMaximitxat = rectVB_passos.origin.y == rectVB_ingredients.origin.y;
    if(!bBlocMaximitxat)
    {
        CGRect rectVB_passos_final = CGRectMake(0, rectVB_ingredients.origin.y, rectVB_ingredients.size.width, rectVB_ingredients.size.height + rectVB_passos.size.height);

        
        self.boto_ulleres_passos.enabled = NO;
        self.boto_ulleres_ingredients.enabled = NO;
        self.liniaUltrafinaSeparadoraBlocs.hidden = YES;
        
        // Animació
        [UIView animateWithDuration:0.5
                              delay:0
                            options:UIViewAnimationOptionCurveEaseInOut animations:^{
            
            self.vista_bloc_passos.frame = rectVB_passos_final;

        } completion:^(BOOL finished) {
            
            self.boto_ulleres_passos.enabled = YES;
            self.boto_ulleres_ingredients.enabled = YES;
            
        }];

    }else{
        
        CGRect rectVB_passos_final = CGRectMake(0, rectVB_ingredients.origin.y + rectVB_ingredients.size.height, rectVB_ingredients.size.width, self.vista_2_blocs.frame.size.height - rectVB_ingredients.size.height);

        self.boto_ulleres_passos.enabled = NO;
        self.boto_ulleres_ingredients.enabled = NO;
        
        // Animació
        [UIView animateWithDuration:0.5
                              delay:0
                            options:UIViewAnimationOptionCurveEaseInOut animations:^{
            
            self.vista_bloc_passos.frame = rectVB_passos_final;

        } completion:^(BOOL finished) {
            
            self.liniaUltrafinaSeparadoraBlocs.hidden = NO;
            self.boto_ulleres_passos.enabled = YES;
            self.boto_ulleres_ingredients.enabled = YES;
            
        }];
        
    }
    
}

-(void)refrescaFoto:(UIImage*)foto
{
    
    self.camera.hidden = YES;
    
    CALayer * l = [self.vistaFoto layer];
    [l setMasksToBounds:YES];
    l.borderWidth = 0;
    
    self.vistaFoto.superview.backgroundColor = UIColor.clearColor;
    
    float width = foto.size.width;
    float height = foto.size.height;
    
    float amplada_definitiva = self.vistaFoto.frame.size.width;
    float alsada_definitiva = self.vistaFoto.frame.size.height;
    
    CGRect rectFoto;
    
    if(height>width)
    {
        
        float ratio = amplada_definitiva/width;
        rectFoto = CGRectMake((amplada_definitiva/2)-((width*ratio)/2), (alsada_definitiva/2)-((height*ratio)/2), width*ratio, height*ratio);
        if(rectFoto.size.height < alsada_definitiva)
        {
            float ratio = alsada_definitiva/rectFoto.size.height;
            rectFoto = CGRectMake((amplada_definitiva/2)-((rectFoto.size.width*ratio)/2), (alsada_definitiva/2)-((rectFoto.size.height*ratio)/2), rectFoto.size.width*ratio, rectFoto.size.height*ratio);
        }
        
    }else{
        
        // width > height
        float ratio = alsada_definitiva/height;
        rectFoto = CGRectMake((amplada_definitiva/2)-((width*ratio)/2), (alsada_definitiva/2)-((height*ratio)/2), width*ratio, height*ratio);
        if(rectFoto.size.width < amplada_definitiva)
        {
            float ratio = amplada_definitiva/rectFoto.size.width;
            rectFoto = CGRectMake((amplada_definitiva/2)-((rectFoto.size.width*ratio)/2), (alsada_definitiva/2)-((rectFoto.size.height*ratio)/2), rectFoto.size.width*ratio, rectFoto.size.height*ratio);
        }
        
    }
    
    if(self.imgvFoto == nil)
    {
        self.imgvFoto = [[UIImageView alloc] initWithFrame:rectFoto];
    }else{
        self.imgvFoto.frame = rectFoto;
    }
    
    self.imgvFoto.image = foto;
    
    if([self.imgvFoto superview] != self.vistaFoto)[self.vistaFoto addSubview:self.imgvFoto];
    
}


- (IBAction)clickAfegirIngredient:(id)sender {
    
    if([self.textfield_preu isFirstResponder])[self.textfield_preu resignFirstResponder];
    if([self.textfield_titol isFirstResponder])[self.textfield_titol resignFirstResponder];
    if([self.textfield_temps isFirstResponder])[self.textfield_temps resignFirstResponder];
    
    ViewControllerAfegirIngredientRecepta *oViewControllerAfegirIngredientRecepta = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"viewControllerAfegirIngredientRecepta"];
    oViewControllerAfegirIngredientRecepta.oViewControllerRecepta = self;
    [self.navigationController pushViewController:oViewControllerAfegirIngredientRecepta animated:YES];
    
}

- (IBAction)clickAfegirPas:(id)sender {
    
    if([self.textfield_preu isFirstResponder])[self.textfield_preu resignFirstResponder];
    if([self.textfield_titol isFirstResponder])[self.textfield_titol resignFirstResponder];
    if([self.textfield_temps isFirstResponder])[self.textfield_temps resignFirstResponder];
    
    ViewControllerPas *oViewControllerPas = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"viewControllerPas"];
    oViewControllerPas.oViewControllerRecepta = self;
    [self.navigationController pushViewController:oViewControllerPas animated:YES];
    
}
#pragma mark Tableview delegate

- (CGFloat)tableView:(UITableView *)aTableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    if(aTableView == self.tableview_ingredients)
    {
        
        return 49;
    
    }else{
        
        float amplada_pantalla = [[UIScreen mainScreen] bounds].size.width;
        NSDictionary *dictEntitat = self.aInstancies_passos[indexPath.row];
        NSString *descripcio = dictEntitat[@"descripcio"];
        NSMutableAttributedString * atributtedString_DESCRIPCIO = [self retornaAttributedStringDescripcio:descripcio];
        int x_tv_descripcio = X_TEXTVIEW_PAS_DESCRIPCIO;
        float ampladaTvDescripcio =  amplada_pantalla - ML_TEXTVIEW_PAS_DESCRIPCIO - x_tv_descripcio;
        float alsadaTvDescripcio = [self textViewHeightForAttributedText:atributtedString_DESCRIPCIO andWidth:ampladaTvDescripcio];
        
        return MAX(alsadaTvDescripcio + 10, 64);
        
    }
    
}

- (UITableViewCell *)tableView:(UITableView *)aTableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    
    float amplada_pantalla = [[UIScreen mainScreen] bounds].size.width;
    
    if(aTableView == self.tableview_ingredients)
    {
        
        NSString *sReuseIdentifier = @"IngredientARecepta_cell_reuse_identifier";
        
        UITableViewCell *cell = [aTableView dequeueReusableCellWithIdentifier:sReuseIdentifier];
        
        // Càlculs comuns
        
        NSDictionary *dictEntitat = self.aInstancies_ingredients[indexPath.row];
        
        NSString *titol = [IncludesCompartits retornaIngredientARecepta:dictEntitat[@"genere"] nom_singular:dictEntitat[@"nom_singular"] nom_plural:dictEntitat[@"nom_plural"] quantitat:dictEntitat[@"quantitat"] unitat:dictEntitat[@"unitat"] utilitza_unitat:dictEntitat[@"utilitza_unitat"]];
        
        int iWidth_labels = amplada_pantalla-30;
        
        NSMutableAttributedString * atributtedString_TITOL = [[NSMutableAttributedString alloc] initWithString:titol];
        [atributtedString_TITOL addAttribute:NSFontAttributeName value:[UIFont systemFontOfSize:16] range: NSMakeRange(0,[titol length])];
        [atributtedString_TITOL addAttribute:NSForegroundColorAttributeName value:[UIColor blackColor] range: NSMakeRange(0,[titol length])];
        
        if(cell == nil)
        {
            
            //NSLog(@"CELL = NIL");
            
            cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:sReuseIdentifier];
            
            UILabel * labelcell_TITOL = [[UILabel alloc]initWithFrame:CGRectMake(14, 4, iWidth_labels, 40)];
            labelcell_TITOL.attributedText = atributtedString_TITOL; // ?¿
            labelcell_TITOL.numberOfLines = 2;
            labelcell_TITOL.backgroundColor = UIColor.clearColor;
            labelcell_TITOL.tag = 1;
            [cell.contentView addSubview:labelcell_TITOL];
            
        }else{
            
            //NSLog(@"CELL <> NIL");
            
            for (UIView * subview in cell.contentView.subviews) {
                
                if([subview class] == [UILabel  class])
                {
                    
                    if(subview.tag == 1)((UILabel*)subview).attributedText = atributtedString_TITOL;
                    
                }

            }
            
        }
        
        return cell;
        
    }else{
        
        NSString *sReuseIdentifier = @"Pas_cell_reuse_identifier";
        
        UITableViewCell *cell = [aTableView dequeueReusableCellWithIdentifier:sReuseIdentifier];
        
        // Càlculs comuns
        
        NSDictionary *dictEntitat = self.aInstancies_passos[indexPath.row];
        
        NSString *numero = dictEntitat[@"numero"];
        NSString *descripcio = dictEntitat[@"descripcio"];
        int num_fotos_adjuntes = [dictEntitat[@"num_fotos_adjuntes"] intValue];
        
        bool bAmagarCamera = num_fotos_adjuntes == 0;
        
        // Text view descripció
        
        NSMutableAttributedString * atributtedString_DESCRIPCIO = [self retornaAttributedStringDescripcio:descripcio];
        
        int x_tv_descripcio = X_TEXTVIEW_PAS_DESCRIPCIO;
        float ampladaTvDescripcio =  amplada_pantalla - ML_TEXTVIEW_PAS_DESCRIPCIO - x_tv_descripcio;
        float alsadaTvDescripcio = [self textViewHeightForAttributedText:atributtedString_DESCRIPCIO andWidth:ampladaTvDescripcio];
        
        NSMutableAttributedString * mutableattributtedstringPosicio = [[NSMutableAttributedString alloc] initWithString:numero];
        [mutableattributtedstringPosicio addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"Helvetica" size:12.0] range: NSMakeRange(0,[numero length])];
        [mutableattributtedstringPosicio addAttribute:NSForegroundColorAttributeName value:[UIColor whiteColor] range: NSMakeRange(0,[numero length])];
        
        CGRect rectDescripcio = CGRectMake(x_tv_descripcio, Y_TEXTVIEW_PAS_DESCRIPCIO, ampladaTvDescripcio, alsadaTvDescripcio);

        
        if(cell == nil)
        {
            
            //NSLog(@"CELL = NIL");
            
            cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:sReuseIdentifier];
            
            UITextView *tvDescripcio = [[UITextView alloc]initWithFrame:rectDescripcio];
            tvDescripcio.backgroundColor = [UIColor clearColor];
            tvDescripcio.attributedText = atributtedString_DESCRIPCIO;
            tvDescripcio.userInteractionEnabled = NO;
            tvDescripcio.tag = 1;
            [cell.contentView addSubview:tvDescripcio];
            
            // Badge número de pas
            
            VistaMedallaPosicio *viewPosicio = [[VistaMedallaPosicio alloc] initWithFrame:CGRectMake(22, 15, 26, 26)];
            viewPosicio.layer.cornerRadius = viewPosicio.frame.size.width/2;
            viewPosicio.layer.masksToBounds = true;
            viewPosicio.backgroundColor = [UIColor blackColor];
            [cell.contentView addSubview:viewPosicio];
            
            UILabel * labelPosicio = [[UILabel alloc]initWithFrame:CGRectMake(22, 15, 26, 26)];
            labelPosicio.tag = 2;
            [labelPosicio setAttributedText:mutableattributtedstringPosicio];
            labelPosicio.numberOfLines = 1;
            labelPosicio.textAlignment = NSTextAlignmentCenter;
            labelPosicio.backgroundColor = [UIColor clearColor];
            [cell.contentView addSubview:labelPosicio];
            
            // Càmera foto
            
            UIImageView *imgvCamera = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"icona_camera_cela_pas"]];
            imgvCamera.frame = CGRectMake(28, 46, 13, 13);
            imgvCamera.hidden = bAmagarCamera;
            imgvCamera.tag = 3;
            [cell.contentView addSubview:imgvCamera];
            
        }else{
            
            //NSLog(@"CELL <> NIL");
            
            for (UIView * subview in cell.contentView.subviews) {
                
                if([subview class] == [UILabel  class])
                {
                    
                    if(subview.tag == 2)((UILabel*)subview).attributedText = mutableattributtedstringPosicio;
                    
                }
                
                if([subview class] == [UITextView  class])
                {
                    
                    if(subview.tag == 1)
                    {
                        ((UITextView*)subview).attributedText = atributtedString_DESCRIPCIO;
                        ((UITextView*)subview).frame = rectDescripcio;
                    }
                    
                }
                
                if([subview class] == [UIImageView  class])
                {
                    
                    if(subview.tag == 3)
                    {
                        ((UIImageView*)subview).hidden = bAmagarCamera;
                    }
                    
                }

            }
            
        }
        
        return cell;
        
    }

    
}

-(NSMutableAttributedString *)retornaAttributedStringDescripcio:(NSString*)descripcio
{
    
    NSMutableAttributedString *retorn = [[NSMutableAttributedString alloc] initWithString:descripcio];
    [retorn addAttribute:NSFontAttributeName value:[UIFont systemFontOfSize:16] range: NSMakeRange(0,[descripcio length])];
    [retorn addAttribute:NSForegroundColorAttributeName value:[UIColor blackColor] range: NSMakeRange(0,[descripcio length])];
    return retorn;

}

- (CGFloat)textViewHeightForAttributedText: (NSAttributedString*)text andWidth: (CGFloat)width {
    
    UITextView *calculationView = [[UITextView alloc] init];
    [calculationView setAttributedText:text];
    CGSize size = [calculationView sizeThatFits:CGSizeMake(width, CGFLOAT_MAX)];
    return size.height;
    
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    
    return 1;
    
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    if(tableView == self.tableview_ingredients)
    {
        
        if(self.aInstancies_ingredients == nil)return 0;
        
        return self.aInstancies_ingredients.count;
        
    }else{
       
        if(self.aInstancies_passos == nil)return 0;
        
        return self.aInstancies_passos.count;
        
    }

}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    [cell setBackgroundColor:[UIColor clearColor]];
    
}

- (void)tableView: (UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
    
    if(tableView == self.tableview_ingredients)
    {

        
        
    }else{
       
        if([self.textfield_preu isFirstResponder])[self.textfield_preu resignFirstResponder];
        if([self.textfield_titol isFirstResponder])[self.textfield_titol resignFirstResponder];
        if([self.textfield_temps isFirstResponder])[self.textfield_temps resignFirstResponder];
        
        NSDictionary *dictEntitat = self.aInstancies_passos[indexPath.row];
        
        ViewControllerPas *oViewControllerPas = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"viewControllerPas"];
        oViewControllerPas.oViewControllerRecepta = self;
        oViewControllerPas.idPas = dictEntitat[@"id"];
        [self.navigationController pushViewController:oViewControllerPas animated:YES];
        
    }
    
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    
    if([self.textfield_preu isFirstResponder])[self.textfield_preu resignFirstResponder];
    if([self.textfield_titol isFirstResponder])[self.textfield_titol resignFirstResponder];
    if([self.textfield_temps isFirstResponder])[self.textfield_temps resignFirstResponder];
    
}

- (nullable UISwipeActionsConfiguration *)tableView:(UITableView *)tableView trailingSwipeActionsConfigurationForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    UIContextualAction *deleteAction = [UIContextualAction
                                        contextualActionWithStyle:UIContextualActionStyleDestructive
                                        title:@"Elimina"
                                        handler:^(UIContextualAction * _Nonnull action,
                                                  __kindof UIView * _Nonnull sourceView,
                                                  void (^ _Nonnull completionHandler)(BOOL))
    {
        
        if(tableView == self.tableview_ingredients)
        {

            NSDictionary *dictEntitat = self.aInstancies_ingredients[indexPath.row];

            bool bEsiPad = [[UIDevice currentDevice].model rangeOfString:@"iPad"].location != NSNotFound;
            
            NSString *ingredientAReceptaAmbArticle = [IncludesCompartits retornaIngredientAReceptaAmbArticle:dictEntitat[@"utilitza_unitat"]
                                                                                                nom_singular:dictEntitat[@"nom_singular"]
                                                                                                  nom_plural:dictEntitat[@"nom_plural"]
                                                                                                   quantitat:dictEntitat[@"quantitat"]
                                                                                                      genere:dictEntitat[@"genere"]];
            
            UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Confirmar"
                                                                           message:[NSString stringWithFormat:@"Estàs segur que vols eliminar %@ de la recepta?",
                                                                                    ingredientAReceptaAmbArticle]
                                                                    preferredStyle:bEsiPad ? UIAlertControllerStyleAlert : UIAlertControllerStyleActionSheet];
            
            UIAlertAction *eliminarAccio = [UIAlertAction actionWithTitle:@"Elimina ingredient de la recepta"
                                                                   style:UIAlertActionStyleDestructive handler:^(UIAlertAction * action) {
                
                AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
                [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
                
                NSString *URL_WS = [NSString stringWithFormat:@"%@/API/eliminaIngredientDeRecepta",WS_HOST];
                
                NSDictionary *parameters = @{
                    @"id_ingredientarecepta": dictEntitat[@"id"]
                };
                
                [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
                    
                    // OK
                    
                    [self refrescaIngredientsRecepta];
                    
                } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
                    
                    // KO
                    [self presentViewController:[IncludesCompartits creaAlerta:@"Error"
                                                                    descripcio:MISSATGE_ERROR_DEFECTE_WS] animated:YES completion:nil];
                    
                }];
                
            }];
            
            [alert addAction:eliminarAccio];
            
            [alert addAction:[UIAlertAction actionWithTitle:@"Cancel·la" style:UIAlertActionStyleCancel handler:^(UIAlertAction * action) {}]];
            
            [self presentViewController:alert animated:YES completion:nil];
            
        }else{
           
            NSDictionary *dictEntitat = self.aInstancies_passos[indexPath.row];

            bool bEsiPad = [[UIDevice currentDevice].model rangeOfString:@"iPad"].location != NSNotFound;
            
            UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Confirmar"
                                                                           message:@"Estàs segur que vols eliminar aquest pas?"
                                                                    preferredStyle:bEsiPad ? UIAlertControllerStyleAlert : UIAlertControllerStyleActionSheet];
            
            UIAlertAction *eliminarAccio = [UIAlertAction actionWithTitle:@"Elimina pas de la recepta"
                                                                   style:UIAlertActionStyleDestructive handler:^(UIAlertAction * action) {
                
                AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
                [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
                
                NSString *URL_WS = [NSString stringWithFormat:@"%@/API/eliminaPas",WS_HOST];
                
                NSDictionary *parameters = @{
                    @"id_pas": dictEntitat[@"id"],
                    @"id_recepta": self.idRecepta
                };
                
                [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
                    
                    // OK
                    
                    [self refrescaPassosRecepta];
                    
                } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
                    
                    // KO
                    [self presentViewController:[IncludesCompartits creaAlerta:@"Error"
                                                                    descripcio:MISSATGE_ERROR_DEFECTE_WS] animated:YES completion:nil];
                    
                }];
                
            }];
            
            [alert addAction:eliminarAccio];
            
            [alert addAction:[UIAlertAction actionWithTitle:@"Cancel·la" style:UIAlertActionStyleCancel handler:^(UIAlertAction * action) {}]];
            
            [self presentViewController:alert animated:YES completion:nil];
            
        }
        
        completionHandler(true);
        
    }];

    UIContextualAction *moureAction;
    
    if(tableView == self.tableview_passos)
    {
        
        moureAction = [UIContextualAction
                       contextualActionWithStyle:UIContextualActionStyleNormal
                       title:@"Moure"
                       handler:^(UIContextualAction * _Nonnull action,
                                 __kindof UIView * _Nonnull sourceView,
                                 void (^ _Nonnull completionHandler)(BOOL))
                       {
            
            NSDictionary *dictEntitat = self.aInstancies_passos[indexPath.row];
            
            int iPosMaxima = (int)self.aInstancies_passos.count;
            
            UIAlertController *alert= [UIAlertController
                                       alertControllerWithTitle:@"Moviment"
                                       message: [NSString stringWithFormat:@"Indica a quina posició vols moure l'element, sent 1 la mínima i %i la màxima",iPosMaxima]
                                       preferredStyle:UIAlertControllerStyleAlert];
            
            UIAlertAction* ok = [UIAlertAction actionWithTitle:@"D'acord" style:UIAlertActionStyleDefault
                                                       handler:^(UIAlertAction * action){
                
                [alert dismissViewControllerAnimated:YES completion:nil];
                
                UITextField *textField = alert.textFields[0];
                
                int iNovaPosicio = [textField.text intValue];
                
                if(iNovaPosicio == [dictEntitat[@"numero"] intValue])
                {
                    
                    NSLog(@"No ens hem mogut");
                    
                    completionHandler(true);
                    
                    return;
                    
                }
                
                if((iNovaPosicio < 1) || (iNovaPosicio > iPosMaxima)){
                    
                    [self presentViewController:[IncludesCompartits creaAlerta:@"Error" descripcio:@"La posició escollida està fora de rang"] animated:YES completion:nil];
                    
                    completionHandler(true);
                    
                    return;
                    
                }
                
                AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
                [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
                
                NSString *URL_WS = [NSString stringWithFormat:@"%@/API/mourePas",WS_HOST];
                
                NSDictionary *parameters = @{
                    @"id_pas": dictEntitat[@"id"],
                    @"id_recepta": self.idRecepta,
                    @"nova_posicio": [NSString stringWithFormat:@"%i", iNovaPosicio]
                };
                
                [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
                    
                    // OK
                    
                    [self refrescaPassosRecepta];
                    
                } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
                    
                }];
                
            }];
            
            UIAlertAction* cancel = [UIAlertAction actionWithTitle:@"Cancel·la" style:UIAlertActionStyleDefault
                                                           handler:^(UIAlertAction * action) {
                
                [alert dismissViewControllerAnimated:YES completion:nil];
                
            }];
            
            alert.view.tintColor = UIColorFromRGB(PALETA_COLOR_VERMELL);
            
            [alert addAction:ok];
            [alert addAction:cancel];
            
            [alert addTextFieldWithConfigurationHandler:^(UITextField *textField) {
                textField.keyboardType = UIKeyboardTypeDecimalPad;
            }];
            
            [self presentViewController:alert animated:YES completion:nil];
            
            completionHandler(true);
            
        }];
        
        moureAction.backgroundColor = UIColorFromRGB(0xFFC236);
        
    }
    
    NSArray *actions;
    
    actions = tableView == self.tableview_passos ? [[NSArray alloc] initWithObjects:deleteAction, moureAction, nil] :
                                                   [[NSArray alloc] initWithObjects:deleteAction, nil];
    
    return [UISwipeActionsConfiguration configurationWithActions:actions];
}


#pragma mark Textfield delegate

-(void)textFieldDidBeginEditing:(UITextField *)textField
{
    
    if(textField == self.textfield_temps)self.textfield_temps.text = [self.textfield_temps.text stringByReplacingOccurrencesOfString:@" minuts" withString:@""];
        
    if(textField == self.textfield_preu)self.textfield_preu.text = [self.textfield_preu.text stringByReplacingOccurrencesOfString:@" €" withString:@""];
        
}

-(BOOL)textFieldShouldEndEditing:(UITextField *)textField
{
    
    if((textField == self.textfield_temps) && (![self.textfield_temps.text isEqualToString:@""]))
    {
        self.bIgnoraRestriccioDigits = true;
        self.textfield_temps.text = [self.textfield_temps.text stringByAppendingString:@" minuts"];
        self.bIgnoraRestriccioDigits = false;
    }
    
    if((textField == self.textfield_preu)  && (![self.textfield_preu.text isEqualToString:@""]))
    {
        self.bIgnoraRestriccioDigits = true;
        self.textfield_preu.text = [self.textfield_preu.text stringByAppendingString:@" €"];
        self.bIgnoraRestriccioDigits = false;
    }
        
    return YES;
    
}

- (void)textFieldDidEndEditing:(UITextField *)textField reason:(UITextFieldDidEndEditingReason)reason
{
    
    if(textField == self.textfield_titol)
    {
        
        if(reason == UITextFieldDidEndEditingReasonCommitted)
        {
            
            // Actualitzem títol
            
            [self actualitzaTitolRecepta];
            
        }

    }
    
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    
    if(textField == self.textfield_titol)
    {
        
        // Actualitzem títol
        
        [self actualitzaTitolRecepta];
        
    }
    
    return YES;

}

-(void)actualitzaTitolRecepta
{
    
    if([self.textfield_titol isFirstResponder])[self.textfield_titol resignFirstResponder];
    
    AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
    
    NSString *URL_WS = [NSString stringWithFormat:@"%@/API/actualitzaTitolRecepta",WS_HOST];
    
    NSDictionary *parameters = @{@"titol": self.textfield_titol.text,
                                 @"id": self.idRecepta
    };
    
    [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        
        [self.oViewControllerReceptes recercaReceptes];
        
    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) { }];
    
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    
    if((textField == self.textfield_preu) ||
       (textField == self.textfield_temps))
    {
        
        if((!self.bIgnoraRestriccioDigits) && (![string isEqualToString:@""]) && ![self totDigits:string])return NO;
        
        if (self.bIgnoraRestriccioDigits)return YES;
        
        // Actualitzem preu / temps
        
        NSString *sValor = [textField.text stringByReplacingCharactersInRange:range withString:string];
        
        NSString *sWS = textField == self.textfield_preu ? @"actualitzaPreuRecepta" : @"actualitzaTempsRecepta";
        
        AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
        [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
        
        NSString *URL_WS = [NSString stringWithFormat:@"%@/API/%@", WS_HOST, sWS];
        
        NSDictionary *parameters = @{@"valor": sValor,
                                     @"id": self.idRecepta
        };
        
        [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
            
        } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) { }];
        
    }
    
    return YES;

}

- (BOOL)totDigits:(NSString*)string
{
    NSCharacterSet* nonNumbers = [[NSCharacterSet decimalDigitCharacterSet] invertedSet];
    NSRange r = [string rangeOfCharacterFromSet: nonNumbers];
    return r.location == NSNotFound && string.length > 0;
}

@end
