//
//  ViewControllerReceptes.m
//  ataula
//
//  Created by Andreu Biosca on 02/04/2020.
//  Copyright © 2020 EGDBE. All rights reserved.
//

#import "ViewControllerReceptes.h"
#import "IncludesCompartits.h"
#import "TabBarControllerPrincipal.h"
#import "CollectionViewCellItem.h"
#import "ViewControllerRecepta.h"

@interface ViewControllerReceptes ()

@end

@implementation ViewControllerReceptes

-(void)viewDidAppear:(BOOL)animated
{
    
    [super viewDidAppear:animated];
    
    TabBarControllerPrincipal *oTabBarControllerPrincipal = (TabBarControllerPrincipal*)self.tabBarController;
    
    [oTabBarControllerPrincipal pintaTextosIIcones];
    
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.title = @"Receptes";
    
    // Vista preloader:
    self.vista_preloader = [IncludesCompartits creaVistaPreloader:self];
    self.vista_preloader.hidden = YES;
    
    // Afegim el reconeixement del gest "long press" al Collection view
    
    UILongPressGestureRecognizer *lpgr
       = [[UILongPressGestureRecognizer alloc]
                     initWithTarget:self action:@selector(handleLongPress:)];
    lpgr.delegate = self;
    lpgr.delaysTouchesBegan = YES;
    lpgr.minimumPressDuration = 1.0;
    [self.collectionview addGestureRecognizer:lpgr];
    
    // Creem el botó d'afegir una nova recepta
    
    UIBarButtonItem *botoAfegir = [[UIBarButtonItem alloc] initWithTitle:@"Afegir" style:UIBarButtonItemStylePlain target:self action:@selector(clicAfegir:)];
    botoAfegir.tintColor = UIColorFromRGB(PALETA_COLOR_VERMELL);
    self.navigationItem.rightBarButtonItem = botoAfegir;
    
    // Configuració del Collection view
    
    self.collectionview.delegate = self;

    self.collectionview.dataSource = self;
    
    [self.collectionview registerClass:[CollectionViewCellItem class] forCellWithReuseIdentifier:@"CollectionViewCellItem"];

    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    
    int ampladaPantalla = [UIScreen mainScreen].bounds.size.width;
    
    bool bEsiPad = [[UIDevice currentDevice].model rangeOfString:@"iPad"].location != NSNotFound;
    
    float amplada_item = ampladaPantalla / (bEsiPad ? 5 : 2);
    
    CGSize size = CGSizeMake(amplada_item, amplada_item);
    
    layout.itemSize = CGSizeMake(size.width, size.height);
    
    [self.collectionview setCollectionViewLayout:layout];
    
    // Cerquem ingredients (en aquest cas sense filtres, es a dir, tots)
    
    [self cercaReceptes:@""];
    
}

-(void)handleLongPress:(UILongPressGestureRecognizer *)gestureRecognizer
{
    
    if (gestureRecognizer.state != UIGestureRecognizerStateBegan)return;
    
    NSIndexPath *indexPath = [self.collectionview indexPathForItemAtPoint:
                              [gestureRecognizer locationInView:self.collectionview]];
    
    if (indexPath != nil){
        
        NSDictionary *dictEntitat = self.aInstancies[indexPath.row];
        
        if([self.searchbar isFirstResponder])[self.searchbar resignFirstResponder];

        bool bEsiPad = [[UIDevice currentDevice].model rangeOfString:@"iPad"].location != NSNotFound;
         
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil
                                                                       message:nil
                                                                preferredStyle:bEsiPad ? UIAlertControllerStyleAlert : UIAlertControllerStyleActionSheet];
        
        NSString *sDataLocal = [NSString stringWithFormat:@"%@-%@-%@", [dictEntitat[@"data_creacio"] substringWithRange:NSMakeRange(8, 2)] , [dictEntitat[@"data_creacio"] substringWithRange:NSMakeRange(5, 2)], [dictEntitat[@"data_creacio"] substringToIndex:4]];
        
        NSString *titol = [dictEntitat[@"titol"] isEqualToString:@""] ? [NSString stringWithFormat:@"Recepta %@", sDataLocal] : dictEntitat[@"titol"];
        
        UIAlertAction *eliminarAccio = [UIAlertAction actionWithTitle:[NSString stringWithFormat:@"Elimina la recepta \"%@\"", titol]
                                                               style:UIAlertActionStyleDestructive handler:^(UIAlertAction * action) {
            
            AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
            [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
            
            NSString *URL_WS = [NSString stringWithFormat:@"%@/API/eliminaRecepta",WS_HOST];
            
            NSDictionary *parameters = @{
                @"id_recepta": dictEntitat[@"id"]
            };
            
            [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
                
                // OK
                
                [self recercaReceptes];
                
            } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
                
                // KO
                
                [self presentViewController:[IncludesCompartits creaAlerta:@"Error"
                                                                descripcio:MISSATGE_ERROR_DEFECTE_WS] animated:YES completion:nil];
                
            }];
            
        }];
        
        [alert addAction:eliminarAccio];
        
        [alert addAction:[UIAlertAction actionWithTitle:@"Cancel·la" style:UIAlertActionStyleCancel handler:^(UIAlertAction * action) {}]];
        
        [self presentViewController:alert animated:YES completion:nil];
        
    }
    
}

-(void)recercaReceptes
{
    
    [self cercaReceptes:self.searchbar.text];
    
}

- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText
{

    [self cercaReceptes:searchText];
    
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar
{
    
    [searchBar resignFirstResponder];
    
}

-(void)cercaReceptes:(NSString*)sCerca
{
    
    self.iIdFilCercaValid++;
    
    __block int iIdFilCercaValid_valid = self.iIdFilCercaValid;
    
    AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
    
    NSString *URL_WS = [NSString stringWithFormat:@"%@/API/cercaReceptes",WS_HOST];
    
    NSDictionary *parameters = @{
        @"cerca": sCerca
    };
    
    [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        
        // OK
        
        if(iIdFilCercaValid_valid == self.iIdFilCercaValid){
            
            NSDictionary *json = responseObject;
            
            self.aInstancies = [[NSMutableArray alloc] init];
            
            for (NSDictionary *dict in json[@"data"]) {
                
                [self.aInstancies addObject:dict];
                
            }
            
            if(iIdFilCercaValid_valid == self.iIdFilCercaValid)[self.collectionview reloadData];
            
            // SIMULAKRE
            // [self collectionView:self.collectionview didSelectItemAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
            
        }
        
    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        
        // KO
        
        [self presentViewController:[IncludesCompartits creaAlerta:@"Error"
                                                        descripcio:MISSATGE_ERROR_DEFECTE_WS] animated:YES completion:nil];
        
    }];
    
}

-(void)clicAfegir:(id)sender
{
    
    if([self.searchbar isFirstResponder])[self.searchbar resignFirstResponder];
    
    self.vista_preloader.hidden = NO;
    UIBarButtonItem *boto = self.navigationItem.rightBarButtonItem;
    self.navigationItem.rightBarButtonItem = nil;
    self.tabBarController.tabBar.userInteractionEnabled = NO;

    AFHTTPSessionManager *manager = [[AFHTTPSessionManager alloc]initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
    
    NSString *URL_WS = [NSString stringWithFormat:@"%@/API/afegirRecepta",WS_HOST];
    
    NSDictionary *parameters = @{};
    
    [manager POST:URL_WS parameters:parameters headers:nil progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        
        // OK
        
        self.vista_preloader.hidden = YES;
        self.navigationItem.rightBarButtonItem = boto;
        self.tabBarController.tabBar.userInteractionEnabled = YES;
        
        NSDictionary *json = responseObject;
        
        [self recercaReceptes];
        
        NSString *id_recepta_afegida = json[@"data"];
        
        ViewControllerRecepta *oViewControllerRecepta = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"viewControllerRecepta"];
        oViewControllerRecepta.oViewControllerReceptes = self;
        oViewControllerRecepta.idRecepta = id_recepta_afegida;
        [self.navigationController pushViewController:oViewControllerRecepta animated:YES];
        
        
    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        
        // KO
        
        self.vista_preloader.hidden = YES;
        self.navigationItem.rightBarButtonItem = boto;
        self.tabBarController.tabBar.userInteractionEnabled = YES;
        
    }];
    
    
    //self.navigationItem.rightBarButtonItem = boto;
    
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    
    if([self.searchbar isFirstResponder])[self.searchbar resignFirstResponder];
    
}

#pragma mark Delegate i Respomder Collection View

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout minimumInteritemSpacingForSectionAtIndex:(NSInteger)section {
    
    return 0.0;
    
}

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout minimumLineSpacingForSectionAtIndex:(NSInteger)section {
    
    return 0.0;
    
}

- (UIEdgeInsets)collectionView:
(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout insetForSectionAtIndex:(NSInteger)section {
    
    return UIEdgeInsetsMake(0,0,0,0);  // top, left, bottom, right
    
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section
{
    
    return self.aInstancies.count;
    
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(nonnull NSIndexPath *)indexPath
{
    
    if([self.searchbar isFirstResponder])[self.searchbar resignFirstResponder];
    
    NSDictionary *dictEntitat = self.aInstancies[indexPath.row];
    ViewControllerRecepta *oViewControllerRecepta = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"viewControllerRecepta"];
    oViewControllerRecepta.idRecepta = dictEntitat[@"id"];
    oViewControllerRecepta.oViewControllerReceptes = self;
    [self.navigationController pushViewController:oViewControllerRecepta animated:YES];

}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
    
    TabBarControllerPrincipal *oTabBarControllerPrincipal = (TabBarControllerPrincipal*)self.tabBarController;
    
    CollectionViewCellItem *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"CollectionViewCellItem" forIndexPath:indexPath];
    
    NSDictionary *dictEntitat = self.aInstancies[indexPath.row];
    
    NSString *sDataLocal = [NSString stringWithFormat:@"%@-%@-%@", [dictEntitat[@"data_creacio"] substringWithRange:NSMakeRange(8, 2)] , [dictEntitat[@"data_creacio"] substringWithRange:NSMakeRange(5, 2)], [dictEntitat[@"data_creacio"] substringToIndex:4]];
    
    NSString *titol = [dictEntitat[@"titol"] isEqualToString:@""] ? [NSString stringWithFormat:@"Recepta %@", sDataLocal] : dictEntitat[@"titol"];
    
    NSString *url_imatge = [dictEntitat[@"foto"] isEqualToString:@""] ? @"" : [NSString stringWithFormat:@"%@%@",WS_HOST,dictEntitat[@"foto"]];
    
    // Foto:
    
    if(oTabBarControllerPrincipal.dictImatgesReceptesCollectionCachejades == nil)oTabBarControllerPrincipal.dictImatgesReceptesCollectionCachejades = [[NSCache alloc]init];
    
    if([url_imatge isEqualToString:@""])
    {
        
        [cell recalculaFoto:[UIImage imageNamed:@"placeholder_recepta.png"]];
        
    }else{
        
        if([oTabBarControllerPrincipal.dictImatgesReceptesCollectionCachejades objectForKey:url_imatge] == nil)
        {
            
            //NSLog(@"🛑 MISS %@",url_imatge);
            
            [cell recalculaFoto:[UIImage imageNamed:@"placeholder_ingredient.png"]];
            
            dispatch_queue_t thread_LOAD_IMG=dispatch_queue_create("thread_LOAD_IMG", DISPATCH_QUEUE_CONCURRENT);
            dispatch_async(thread_LOAD_IMG, ^{

                NSIndexPath *snapshot_indexpath = [NSIndexPath indexPathForRow:indexPath.row inSection:indexPath.section];
                NSData * imgdata = [NSData dataWithContentsOfURL:[NSURL URLWithString:url_imatge]];
                if (imgdata != nil)[oTabBarControllerPrincipal.dictImatgesReceptesCollectionCachejades setObject:imgdata forKey:url_imatge];
                    
                dispatch_async(dispatch_get_main_queue(), ^{
                    
                    // UI CHANGES
                    
                    if(imgdata != nil)[collectionView reloadItemsAtIndexPaths:@[snapshot_indexpath]];

                });
            });
            
        }else{
            
            //NSLog(@"✅ HIT %@",url_imatge);
            
            [cell recalculaFoto:[UIImage imageWithData:[oTabBarControllerPrincipal.dictImatgesReceptesCollectionCachejades objectForKey:url_imatge]]];
            
        }
        
    }

    // (titol)
    
    cell.label_titol.text = titol;

    return cell;
    
}

@end
